from collections.abc import Mapping

import requests

from .types import ItemData, RecipesData, UsesData, LineageData


Params = Mapping[str, int | bool | str | None]


class Infinibrowser:
    """
    Infinibrowser Client
    """

    # Base URL for the API
    API_URL = "https://infinibrowser.wiki/"

    @classmethod
    def _get_request(cls, path: str, params: Params | None = None):
        url = f"{cls.API_URL}{path}"
        response = requests.get(url, params=params)
        response.raise_for_status()
        return response.json()

    @classmethod
    def get_item(cls, id: str):
        """
        Get information about the item
        """

        path = "/api/item"
        params = {"id": id}

        data = cls._get_request(path=path, params=params)

        return ItemData(**data)

    @classmethod
    def get_recipes(cls, id: str, offset=0):
        """
        Get recipes for the item
        """

        path = "/api/recipes"
        params = {"id": id, "offset": offset}

        data = cls._get_request(path=path, params=params)

        return RecipesData(**data)

    @classmethod
    def get_uses(cls, id: str, offset=0):
        """
        Get uses for the item
        """

        path = "/api/uses"
        params = {"id": id, "offset": offset}

        data = cls._get_request(path=path, params=params)

        return UsesData(**data)

    @classmethod
    def get_lineage(cls, id: str):
        """
        Get lineage for the item
        """

        path = "/api/recipe"
        params = {"id": id}

        data = cls._get_request(path=path, params=params)

        return LineageData(**data)



from .items import Element, ItemData, Recipe, RecipesData, Use, UsesData, Step, Lineage, LineageData

__all__ = [
    "Element",
    "ItemData",
    "Recipe",
    "RecipesData",
    "Use",
    "UsesData",
    "Step",
    "Lineage",
    "LineageData",
]



from typing import Literal
from pydantic import BaseModel


class Element(BaseModel):
    id: str
    emoji: str


class ItemData(BaseModel):
    id: int
    text: str
    emoji: str
    use_count: int
    recipe_count: int
    depth: int


Recipe = tuple[Element, Element]


class RecipesData(BaseModel):
    total: int
    recipes: list[Recipe]


class Use(BaseModel):
    pair: Element
    result: Element


class UsesData(BaseModel):
    total: int
    uses: list[Use]


class Step(BaseModel):
    a: Element
    b: Element
    result: Element


Lineage = list[Step]


class LineageData(BaseModel):
    steps: Lineage
    missing: dict[str, str | Literal["loop"]]


__all__ = [
    "Element",
    "ItemData",
    "Recipe",
    "RecipesData",
    "Use",
    "UsesData",
    "Step",
    "Lineage",
    "LineageData",
]
